use v5.38;
use feature 'class';
no warnings 'experimental::class';

class Archive::SCS::Mountable 0.01;


=head1 NAME

Archive::SCS::Mountable - Something that can be mounted by Archive::SCS

=head1 DESCRIPTION

Represents an SCS archive file or a similarly mountable entity.

This may become a role in future.

=head1 AUTHOR

L<nautofon|https://github.com/nautofon>

=head1 COPYRIGHT

This software is copyright (c) 2024 by nautofon.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.
